@@ -2,28 +2,49 @@
 
 if(getRversion() < "2.15")
     paste0 <- function(...) paste(..., sep = '')
+if(getRversion() < "3.3")
+    strrep <- function (x, times) { ## (x, times) must be "recycled"
+	if((lx <- length(x)) < (lt <- length(times)))
+	    x <- rep_len(x, lt)
+	else if(lt < lx)
+	    times <- rep_len(times, lx)
+	vapply(seq_along(x),
+	       function(i) paste(rep.int(x[i], times[i]), collapse = ""), "")
+    }
 
-mpfr <- function(x, precBits, base = 10, rnd.mode = c('N','D','U','Z','A'))
+mpfr <- function(x, precBits, ...) UseMethod("mpfr")
+
+mpfr.mpfr <- function(x, precBits, rnd.mode = c('N','D','U','Z','A'), ...)
+    roundMpfr(x, precBits=precBits, rnd.mode=rnd.mode)
+
+mpfr.bigz <- function(x, precBits, ...) {
+    if(missing(precBits)) precBits <- max(2L, frexpZ(x)$exp)
+    if(getOption("verbose"))
+	warning("mpfr(<bigz>) --> .bigz2mpfr() [not efficiently via character]")
+    ..bigz2mpfr(x, precBits)
+}
+
+mpfr.bigq <- function(x, precBits, ...) {
+    if(missing(precBits)) precBits <- getPrec(x)#-> warning
+    if(getOption("verbose"))
+	warning("mpfr(<bigq>) --> .bigq2mpfr() [not efficiently via character]")
+    ..bigq2mpfr(x, precBits)
+}
+
+mpfr.default <- function(x, precBits, base = 10, rnd.mode = c('N','D','U','Z','A'),
+                         scientific = NA, ...)
 {
-    if(is.character(x))
-	stopifnot(length(base) == 1, 2 <= base, base <= 36)
+    if(is.ch <- is.character(x))
+	stopifnot(length(base) == 1, 2 <= base, base <= 62)
     stopifnot(is.character(rnd.mode <- toupper(rnd.mode)))
     rnd.mode <- match.arg(rnd.mode)
 
     if(is.raw(x)) { # is.raw() is faster
 	stopifnot(missing(precBits) || precBits >= 2)
-	if(inherits(x, "bigz")) {
-	    if(missing(precBits)) precBits <- max(2L, frexpZ(x)$exp)
-	    if(getOption("verbose"))
-	       warning("mpfr(<bigz>) --> .bigz2mpfr()")# via character
-	    return(..bigz2mpfr(x, precBits))
-	} else if(inherits(x, "bigq")) {
-	    if(missing(precBits)) precBits <- getPrec(x)#-> warning
-	    if(getOption("verbose"))
-		warning("mpfr(<bigq>) --> .bigq2mpfr()")# via character
-	    return(..bigq2mpfr(x, precBits))
-	}
+	## else warning("unrecognized raw 'x'") # <- ?? {see use in ../tests/create.R }
+        ## {but 'raw' is treated below}
     } ## else
+
     if(missing(precBits)) {
 	precBits <- getPrec(x, base = base, doNumeric = FALSE)
     }
@@ -33,7 +54,7 @@ mpfr <- function(x, precBits, base = 10, rnd.mode = c('N','D','U','Z','A'))
     ml <-
 	if(is.numeric(x) || is.logical(x) || is.raw(x))
 	    .Call(d2mpfr1_list, x, precBits, rnd.mode)
-	else if(is.character(x))
+	else if(is.ch)
 	    .Call(str2mpfr1_list,x, precBits, base, rnd.mode)
 	else stop("invalid 'x'. Must be numeric (logical, raw) or character")
     if(is.array(x)) {
@@ -43,7 +64,7 @@ mpfr <- function(x, precBits, base = 10, rnd.mode = c('N','D','U','Z','A'))
 	    Dimnames = if(is.null(dn)) vector("list", length(dim)) else dn)
     }
     else new("mpfr", ml)
-}
+} ## mpfr.default()
 
 .mpfr <- function(x, precBits)
     new("mpfr", .Call(d2mpfr1_list, x, precBits, "N"))
@@ -113,27 +134,28 @@ mpfrImport <- function(mxp) {
     new("mpfr", m1)
 }
 
-.mpfr2str <- function(x, digits = NULL) {
-    stopifnot(is.null(digits) ||
-	      (is.numeric(digits) && digits >= 1))
+.mpfr2str <- function(x, digits = NULL, base = 10L) {
     ##	digits = NULL : use as many digits "as needed"
-    .Call(mpfr2str, x, digits)
+    stopifnot(is.null(digits) ||
+	      (is.numeric(digits) && digits >= 1),
+	      is.numeric(base), length(base) == 1, base >= 2)
+    .Call(mpfr2str, x, digits, base)
 }
 
 formatMpfr <-
     function(x, digits = NULL, trim = FALSE, scientific = NA,
-	     showNeg0 = TRUE,
+             base = 10, showNeg0 = TRUE,
 	     big.mark = "", big.interval = 3L,
-	     small.mark = "", small.interval = 5L, decimal.mark = ".",
+	     small.mark = "", small.interval = 5L,
+             decimal.mark = ".",
+             exponent.char = if(base <= 14) "e" else if(base <= 36) "E" else "|e",
 	     zero.print = NULL, drop0trailing = FALSE, ...)
 {
-    stopifnot(is.null(digits) ||
-	      (is.numeric(digits) && digits >= 1))
     ##	digits = NULL : use as many digits "as needed"
+    ff <- .mpfr2str(x, digits, base=base)
 
-    ff <- .mpfr2str(x, digits)
     isNum <- ff$finite	## ff$finite == is.finite(x)
-    i0 <- ff$is.0	## == mpfr.is.0(x)
+    i0 <- ff$is.0	## == mpfrIs0(x)
     ex <- ff$exp ## the *decimal* exp : one too large *unless* x == 0
     r  <- ff$str
     if(is.null(digits)) digits <- nchar(r)
@@ -154,10 +176,10 @@ formatMpfr <-
 	hasMinus <- sign(x) == -1
     }
 
-    if(!all(isNum))
+    if(!all(isNum)) ## "@Inf@", "@NaN@", ...
 	r[!isNum] <- gsub("@", '', r[!isNum], fixed=TRUE)
 
-    ## (maybe) add decimal point
+    ##' (maybe) add decimal point after position  k
     patch <- function(str, k)
 	paste(substr   (str, 1L, k),
 	      substring(str, k+1L), sep = decimal.mark)
@@ -170,18 +192,20 @@ formatMpfr <-
     hasE <- { if(is.logical(scientific)) scientific else
 	      isNum & (Ex < -4 + scientific | Ex > digits) }
 
-    if(any(hasE)) {
+    if(aE <- any(ii <- isNum & hasE)) {
+        ii <- which(ii)
 	i. <- 1L + hasMinus
-	ii <- isNum & hasE
 	r[ii] <- patch(r[ii], i.[ii])
-	## FIXME : if this is correct, make it simpler :
-	## r[hasE] <- paste(r[hasE], as.character(Ex[hasE]), sep = "e")
-	r[ii] <- paste(r[ii], as.character(Ex[ii]), sep = "e")
+	if(drop0trailing)
+	    ## drop 0's only *after* (and together with!) decimal mark:
+	    r[ii] <- sub(paste0(decimal.mark, "0+$"), "", r[ii])
+	r[ii] <- paste(r[ii], as.character(Ex[ii]), sep = exponent.char)
     }
-    if(!all(hasE)) { ## "non-scientific" i.e. without final  e<nn> :
+    use.prettyN <- (base <= 14 && (!aE || exponent.char == "e"))
+    if(non.sci <- !all(hasE)) { ## "non-scientific" i.e. without final  e<nn> :
 	ii <- isNum & !hasE
-	## iNeg <- ex <= 0	 & ii ## i.e., ex	 in {0,-1,-2,-3}
-	## iPos <- ex > 0	 & ii ## i.e., ex	 in {1,2..., digits}
+	## iNeg <- ex <= 0 & ii ## i.e., ex	 in {0,-1,-2,-3}
+	## iPos <- ex >  0 & ii ## i.e., ex	 in {1,2..., digits}
 	iNeg <- Ex <  0	 & ii ## i.e., ex	 in {0,-1,-2,-3}
 	iPos <- Ex >= 0	 & ii ## i.e., ex	 in {1,2..., digits}
 
@@ -205,12 +229,21 @@ formatMpfr <-
 	if(any(iPos)) ## "xy.nnnn" :
 	    r[iPos] <- patch(r[iPos], (hasMinus + Ex+1L)[iPos])
     }
-    r <- prettyNum(r, big.mark = big.mark, big.interval = big.interval,
-		   small.mark = small.mark,
-		   small.interval = small.interval,
-		   decimal.mark = decimal.mark,
-		   zero.print = zero.print, drop0trailing = drop0trailing,
-		   preserve.width = if (trim) "individual" else "common")
+    if(use.prettyN)
+        r <- prettyNum(r, big.mark = big.mark, big.interval = big.interval,
+                       small.mark = small.mark,
+                       small.interval = small.interval,
+                       decimal.mark = decimal.mark,
+                       zero.print = zero.print, drop0trailing = drop0trailing,
+                       preserve.width = if (trim) "individual" else "common")
+    else {
+	if(non.sci && drop0trailing)
+	    ## drop 0's only *after* (and together with!) decimal mark:
+	    r <- sub(paste0(decimal.mark, "0+$"), "", r)
+	if(!missing(big.mark) || !missing(big.interval) || !missing(small.interval) ||
+	    !missing(small.mark) || !missing(big.interval) || !missing(zero.print))
+	    warning("with base >= 15 or 'exponent.char != \"e\", cannot use prettyNum()")
+    }
     if(is.null(d <- dim(x))) r
     else array(r, dim=d, dimnames = dimnames(x))
 }
