@@ -27,9 +27,9 @@ pnorm <- function (q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
 {
     if(is.numeric(q) && is.numeric(mean) && is.numeric(sd))
 	stats__pnorm(q, mean, sd, lower.tail=lower.tail, log.p=log.p)
-    else if(is(q, "mpfr") || is(mean, "mpfr") || is(sd, "mpfr")) {
+    else if((q.mp <- is(q, "mpfr")) || is(mean, "mpfr") || is(sd, "mpfr")) {
 	stopifnot(length(lower.tail) == 1, length(log.p) == 1)
-	rr <- q <- (as(q, "mpfr") - mean) / sd
+	rr <- q <- ((if(q.mp) q else as(q, "mpfr")) - mean) / sd
 	if(any(neg <- (q < 0))) ## swap those:	Phi(-z) = 1 - Phi(z)
 	    rr[neg] <- pnorm(-q[neg], lower.tail = !lower.tail, log.p=log.p)
 	if(any(pos <- !neg)) {
@@ -60,9 +60,7 @@ pnorm <- function (q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
 dnorm <- function (x, mean = 0, sd = 1, log = FALSE) {
     if(is.numeric(x) && is.numeric(mean) && is.numeric(sd))
 	stats__dnorm(x, mean, sd, log=log)
-    else if((x.mp <- is(x, "mpfr")) |
-	    (m.mp <- is(mean, "mpfr")) |
-	    (s.mp <- is(sd, "mpfr"))) {
+    else if((x.mp <- is(x, "mpfr")) || is(mean, "mpfr") || (s.mp <- is(sd, "mpfr"))) {
 	## stopifnot(length(log) == 1)
 	prec <- pmax(53, getPrec(x), getPrec(mean), getPrec(sd))
 	if(!x.mp) x <- mpfr(x, prec)
@@ -79,7 +77,7 @@ dnorm <- function (x, mean = 0, sd = 1, log = FALSE) {
 dpois <- function (x, lambda, log = FALSE) {
     if(is.numeric(x) && is.numeric(lambda)) ## standard R
 	stats__dpois(x, lambda, log=log)
-    else if((l.mp <- is(lambda, "mpfr")) | (x.mp <- is(x, "mpfr"))) {
+    else if((l.mp <- is(lambda, "mpfr")) || (x.mp <- is(x, "mpfr"))) {
 	prec <- pmax(53, getPrec(lambda), getPrec(x))
 	if(!l.mp) lambda <- mpfr(lambda, prec)
 	if(!x.mp) x <- mpfr(x, prec)
@@ -112,7 +110,7 @@ dbinom <- function (x, size, prob, log = FALSE) {
 
 ## zeta()
 zeta <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr") # keep "mpfrArray"
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["zeta"]])
     x
 }
@@ -133,14 +131,14 @@ Bernoulli <- function(k, precBits = 128) {
 
 ## eint() "Exponential integral"
 Ei <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr") # keep "mpfrArray"
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["Eint"]])
     x
 }
 
 ## Li_2() the dilogarithm
 Li2 <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr") # keep "mpfrArray"
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["Li2"]])
     x
 }
@@ -150,40 +148,40 @@ Li2 <- function(x) {
 ## j0, j1, jn
 ## y0, y1, yn
 j0 <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr") # keep "mpfrArray"
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["j0"]])
     x
 }
 j1 <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr")
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["j1"]])
     x
 }
 y0 <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr")
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["y0"]])
     x
 }
 y1 <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr")
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["y1"]])
     x
 }
 
 Ai <- function(x) {
-    x <- as(x, "mpfr")
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr")
     x@.Data[] <- .Call(Math_mpfr, x, .Math.codes[["Ai"]])
     x
 }
 
-jn <- function(n, x) {
-    x <- as(x, "mpfr")
-    x@.Data[] <- .Call(R_mpfr_jn, x, n)
+jn <- function(n, x, rnd.mode = c('N','D','U','Z','A')) {
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr")
+    x@.Data[] <- .Call(R_mpfr_jn, x, n, match.arg(rnd.mode))
     x
 }
-yn <- function(n, x) {
-    x <- as(x, "mpfr")
-    x@.Data[] <- .Call(R_mpfr_yn, x, n)
+yn <- function(n, x, rnd.mode = c('N','D','U','Z','A')) {
+    if(!inherits(x, "mpfr")) x <- as(x, "mpfr")
+    x@.Data[] <- .Call(R_mpfr_yn, x, n, match.arg(rnd.mode))
     x
 }
 
@@ -192,75 +190,67 @@ yn <- function(n, x) {
 
 ## We want to automatically construct the methods needed:
 ## But atan2() as argument list and  signature	(y, x)
-## where  beta() has  (a,b)
+## where  beta() and lbeta()  have  (a,b) --> cannot treat them identically;
+## and treat atan2() speparately
 
-mpfrMath2setMeth.y.x <- function(fname, Csub) {
-    stopifnot(existsFunction(fname),
-	      is.character(fname), length(fname) == 1,
-	      is.character(Csub ), length(Csub ) == 1)
+## NB: atan2(), beta() and lbeta() all have implicitGeneric()s in methods with no '...'
+## ==  ---> canNOT have 3rd argument : rnd.mode = c('N','D','U','Z','A')
+##     ---> using   "N"  instead of    match.arg(rnd.mode)
+setMethod("atan2", signature(y = "mpfr", x = "mpfr"),
+          function(y, x) new("mpfr", .Call(R_mpfr_atan2, y, x, "N")))
+setMethod("atan2", signature(y = "mpfr", x = "numeric"),
+          function(y, x) new("mpfr", .Call(R_mpfr_atan2, y, .mpfr(x, 128L), "N")))
+setMethod("atan2", signature(y = "numeric", x = "mpfr"),
+          function(y, x) new("mpfr", .Call(R_mpfr_atan2, .mpfr(y, 128L), x, "N")))
+setMethod("atan2", signature(y = "mpfr", x = "ANY"),
+          function(y, x) new("mpfr", .Call(R_mpfr_atan2, y, as(x, "mpfr"), "N")))
+setMethod("atan2", signature(y = "ANY", x = "mpfr"),
+          function(y, x) new("mpfr", .Call(R_mpfr_atan2, as(y, "mpfr"), x, "N")))
 
-    setMethod(fname, signature(y = "mpfr", x = "mpfr"),
-	      function(y, x) new("mpfr", .Call(Csub, y, x)))
-    setMethod(fname, signature(y = "mpfr", x = "numeric"),
-	      function(y, x) new("mpfr", .Call(Csub, y, .mpfr(x, 128L))))
-    setMethod(fname, signature(y = "numeric", x = "mpfr"),
-	      function(y, x) new("mpfr", .Call(Csub, .mpfr(y, 128L), x)))
-    setMethod(fname, signature(y = "mpfr", x = "ANY"),
-	      function(y, x) new("mpfr", .Call(Csub, y, as(x, "mpfr"))))
-    setMethod(fname, signature(y = "ANY", x = "mpfr"),
-	      function(y, x) new("mpfr", .Call(Csub, as(y, "mpfr"), x)))
-
-    setMethod(fname, signature(y = "mpfrArray", x = "mpfrArray"),
-	      function(y, x) {
-		  if(dim(x) != dim(y))
-		      stop("array dimensions differ")
-		  x@.Data[] <- .Call(Csub, y, x)
-		  x
-	      })
-    setMethod(fname, signature(y = "mpfrArray", x = "ANY"),
-	      function(y, x) {
-		  if(length(y) %% length(x) != 0)
-		      stop("length of first argument (array) is not multiple of the second argument's one")
-		  y@.Data[] <- .Call(Csub, y, if(is.numeric(x))
-				     .mpfr(x, 128L) else as(x, "mpfr"))
-		  y
-	      })
-    setMethod(fname, signature(y = "ANY", x = "mpfrArray"),
-	      function(y, x) {
-		  if(length(x) %% length(y) != 0)
-		      stop("length of second argument (array) is not multiple of the first argument's one")
-		  x@.Data[] <- .Call(Csub, if(is.numeric(y))
-				     .mpfr(y, 128L) else as(y, "mpfr"), x)
-		  x
-	      })
-
-} ## end{mpfrMath2setMeth.y.x}
-
-## atan2():
-mpfrMath2setMeth.y.x("atan2", "R_mpfr_atan2")
-
-mpfrMath2setMeth.a.b <- function(fname, Csub) {
-    stopifnot(existsFunction(fname),
-	      is.character(fname), length(fname) == 1,
-	      is.character(Csub ), length(Csub ) == 1)
+setMethod("atan2", signature(y = "mpfrArray", x = "mpfrArray"),
+          function(y, x) {
+              if(dim(x) != dim(y))
+                  stop("array dimensions differ")
+              x@.Data[] <- .Call(R_mpfr_atan2, y, x, "N")
+              x
+          })
+setMethod("atan2", signature(y = "mpfrArray", x = "ANY"),
+          function(y, x) {
+              if(length(y) %% length(x) != 0)
+                  stop("length of first argument (array) is not multiple of the second argument's one")
+              y@.Data[] <- .Call(R_mpfr_atan2, y, if(is.numeric(x))
+                  .mpfr(x, 128L) else as(x, "mpfr"), "N")
+              y
+          })
+setMethod("atan2", signature(y = "ANY", x = "mpfrArray"),
+          function(y, x) {
+              if(length(x) %% length(y) != 0)
+                  stop("length of second argument (array) is not multiple of the first argument's one")
+              x@.Data[] <- .Call(R_mpfr_atan2, if(is.numeric(y))
+                  .mpfr(y, 128L) else as(y, "mpfr"), x, "N")
+              x
+          })
 
+## Using  "macro"  {instead of previous aux. function  mpfrMath2setMeth.a.b() :
+for (ff in list(c("beta",  "R_mpfr_beta"),
+                c("lbeta", "R_mpfr_lbeta"))) eval(substitute(
+{
     setMethod(fname, signature(a = "mpfr", b = "mpfr"),
-	      function(a, b) new("mpfr", .Call(Csub, a, b)))
+	      function(a, b) new("mpfr", .Call(Csub, a, b, "N")))
     setMethod(fname, signature(a = "mpfr", b = "numeric"),
-	      function(a, b) new("mpfr", .Call(Csub, a, .mpfr(b, 128L))))
+	      function(a, b) new("mpfr", .Call(Csub, a, .mpfr(b, 128L), "N")))
     setMethod(fname, signature(a = "numeric", b = "mpfr"),
-	      function(a, b) new("mpfr", .Call(Csub, .mpfr(a, 128L), b)))
+	      function(a, b) new("mpfr", .Call(Csub, .mpfr(a, 128L), b, "N")))
     setMethod(fname, signature(a = "mpfr", b = "ANY"),
-	      function(a, b) new("mpfr", .Call(Csub, a, as(b, "mpfr"))))
+	      function(a, b) new("mpfr", .Call(Csub, a, as(b, "mpfr"), "N")))
     setMethod(fname, signature(a = "ANY", b = "mpfr"),
-	      function(a, b) new("mpfr", .Call(Csub, as(a, "mpfr"), b)))
-
+	      function(a, b) new("mpfr", .Call(Csub, as(a, "mpfr"), b, "N")))
 
     setMethod(fname, signature(a = "mpfrArray", b = "mpfrArray"),
 	      function(a, b) {
 		  if(dim(b) != dim(a))
 		      stop("array dimensions differ")
-		  b@.Data[] <- .Call(Csub, a, b)
+		  b@.Data[] <- .Call(Csub, a, b, "N")
 		  b
 	      })
     setMethod(fname, signature(a = "mpfrArray", b = "ANY"),
@@ -268,7 +258,7 @@ mpfrMath2setMeth.a.b <- function(fname, Csub) {
 		  if(length(a) %% length(b) != 0)
 		      stop("length of first argument (array) is not multiple of the second argument's one")
 		  a@.Data[] <- .Call(Csub, a, if(is.numeric(b))
-				     .mpfr(b, 128L) else as(b, "mpfr"))
+				     .mpfr(b, 128L) else as(b, "mpfr"), "N")
 		  a
 	      })
     setMethod(fname, signature(a = "ANY", b = "mpfrArray"),
@@ -276,48 +266,42 @@ mpfrMath2setMeth.a.b <- function(fname, Csub) {
 		  if(length(b) %% length(a) != 0)
 		      stop("length of second argument (array) is not multiple of the first argument's one")
 		  b@.Data[] <- .Call(Csub, if(is.numeric(a))
-				     .mpfr(a, 128L) else as(a, "mpfr"), b)
+				     .mpfr(a, 128L) else as(a, "mpfr"), b, "N")
 		  b
 	      })
 
-} ## end{mpfrMath2setMeth.a.b}
-
-mpfrMath2setMeth.a.b("beta",  "R_mpfr_beta")
-mpfrMath2setMeth.a.b("lbeta", "R_mpfr_lbeta")
-
-rm(mpfrMath2setMeth.y.x,
-   mpfrMath2setMeth.a.b)
+}, list(fname = ff[[1]], Csub = as.symbol(ff[[2]]))))
 
 
 ## hypot()
-hypot <- function(x,y) {
+hypot <- function(x,y, rnd.mode = c('N','D','U','Z','A')) {
     if(is(x, "mpfrArray") || is.array(x)) {
 	if(is.array(x)) x <- mpfrArray(x, 128L, dim=dim(x), dimnames(x))
 	if(is.array(y)) y <- mpfrArray(y, 128L, dim=dim(y), dimnames(y))
 	if(is(y, "mpfrArray")) {
 	    if(dim(x) != dim(y))
 		stop("array dimensions differ")
-	    x@.Data[] <- .Call(R_mpfr_hypot, x, y)
+	    x@.Data[] <- .Call(R_mpfr_hypot, x, y, match.arg(rnd.mode))
 	    x
 	} else { ## y is not (mpfr)Array
 	    if(length(x) %% length(y) != 0)
 		stop("length of first argument (array) is not multiple of the second argument's one")
-	    x@.Data[] <- .Call(R_mpfr_hypot, x, as(y, "mpfr"))
+	    x@.Data[] <- .Call(R_mpfr_hypot, x, as(y, "mpfr"), match.arg(rnd.mode))
 	    x
 	}
     } else if(is(y, "mpfrArray")) {
 	if(length(y) %% length(x) != 0)
 	    stop("length of second argument (array) is not multiple of the first argument's one")
-	y@.Data[] <- .Call(R_mpfr_hypot, as(x, "mpfr"), y)
+	y@.Data[] <- .Call(R_mpfr_hypot, as(x, "mpfr"), y, match.arg(rnd.mode))
 	y
     }
     else
-	new("mpfr", .Call(R_mpfr_hypot, as(x, "mpfr"), as(y, "mpfr")))
+	new("mpfr", .Call(R_mpfr_hypot, as(x, "mpfr"), as(y, "mpfr"), match.arg(rnd.mode)))
 }
 
 ## The Beta(a,b)  Cumulative Probabilities are exactly computable for *integer* a,b:
 pbetaI <- function(q, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE,
-		   precBits = NULL)
+		   precBits = NULL, rnd.mode = c('N','D','U','Z','A'))
 {
     stopifnot(length(shape1) == 1, length(shape2) == 1,
 	      is.whole(shape1), is.whole(shape2),
@@ -344,7 +328,7 @@ pbetaI <- function(q, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE,
     if(pr.x < precBits || !is(q, "mpfr"))
 	q <- mpfr(q, precBits=precBits)
 
-    mpfr1 <- list(.Call(const_asMpfr, 1, 16L)) # as prototype for vapply()
+    mpfr1 <- list(.Call(const_asMpfr, 1, 16L, "N")) # as prototype for vapply()
     F <- if(log.p) log else identity
 
     if(lower.tail) {
@@ -364,5 +348,5 @@ pbetaI <- function(q, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE,
 	new("mpfr",
 	    vapply(q, FUN.x, mpfr1))),
 	      ## reduce the precision, in order to not "claim wrongly":
-	      precBits=precBits)
+	      precBits=precBits, match.arg(rnd.mode))
 }
